/* 保养工单
 * @Author: Li Hanlin
 * @Date: 2023-01-19 09:53:59
 * @Last Modified by: Li Hanlin
 * @Last Modified time: 2023-01-19 10:03:55
 */

import * as React from 'react';
import { useState, useMemo, useRef } from 'react';
import DrawerPro from '@/components/DrawerPro';
import AutoTable from '@/components/AutoTable';
import PremButton from '@/components/PremButton';
import getcolumns from './columns';
import { useRequest } from 'ahooks';
import { doFetch } from '@/utils/doFetch';
import AutoTables from '@/components/AutoTable/mtable';
import { Radio, InputNumber, Input, Divider } from 'antd';
import InitForm from '@/components/InitForm';
import getDetailColumns from './detailColumns';
import DetailNode from '@/components/DetailNode';

function WorkOrder(props) {
  const actionRef = useRef(),
    formRef = useRef();
  const [drawer, setdrawer] = useState({
      open: false,
    }),
    [activeTabKey, setactiveTabKey] = useState('1');

  const { run, loading, runAsync } = useRequest(doFetch, {
    manual: true,
    onSuccess: (res, params) => {
      if (res?.code == '0000') {
        actionRef?.current?.reload();
        setdrawer((s) => ({
          ...s,
          open: false,
        }));
      }
    },
  });

  const detail = (text, row, _, action) => {
    return (
      <PremButton
        btn={{
          size: 'small',
          type: 'link',
          onClick: () => {
            setdrawer((s) => ({
              ...s,
              open: true,
              item: row,
              title: '详情',
              val: 'detail',
              title: '详细信息',
            }));
          },
        }}
      >
        详情
      </PremButton>
    );
  };

  const detailaddonColumns = useMemo(() => {
    if (activeTabKey == 1) {
      if (drawer?.type == 'yz') {
        const columnsc = [
          {
            title: '保养项目',
            dataIndex: 'maintainItemName',
            key: 'maintainItemName',
            hideInForm: true,
          },
          {
            title: '部位',
            dataIndex: 'maintainPosition',
            key: 'maintainPosition',
            hideInForm: true,
          },
          {
            title: '保养方法',
            dataIndex: 'maintainMethod',
            key: 'maintainMethod',
            hideInForm: true,
          },
          {
            title: '结果',
            dataIndex: 'judgeResultName',
            key: 'judgeResultName',
            hideInForm: true,
          },
          {
            title: '下限值',
            dataIndex: 'lowerLimit',
            key: 'lowerLimit',
            hideInForm: true,
          },
          {
            title: '上限值',
            dataIndex: 'upperLimit',
            key: 'upperLimit',
            hideInForm: true,
          },
          {
            title: '备注',
            dataIndex: 'remark',
            key: 'remark',
            hideInForm: true,
          },
        ];
        return [
          {
            title: '工单信息',
            valueType: 'split',
          },
          {
            title: '保养单号',
            dataIndex: 'taskNo',
            key: 'taskNo',
          },
          {
            title: '设备编号',
            dataIndex: 'equipmentNo',
            key: 'equipmentNo',
          },
          {
            title: '设备名称',
            dataIndex: 'equipmentName',
            key: 'equipmentName',
          },
          {
            title: '保养类型',
            dataIndex: 'maintainTypeName',
            key: 'maintainTypeName',
          },

          {
            title: '保养频次',
            dataIndex: 'maintainFrequencyName',
            key: 'maintainFrequency',
          },
          {
            title: '保养截止日期',
            dataIndex: 'planMaintainDate',
            key: 'planMaintainDate',
          },
          {
            title: '接单时间',
            dataIndex: 'maintainStartTime',
            key: 'maintainStartTime',
          },
          {
            title: '完成保养时间',
            dataIndex: 'maintainEndTime',
            key: 'maintainEndTime',
          },
          {
            title: '保养项目',
            valueType: 'split',
          },
          {
            dataIndex: 'itemList',
            key: 'itemList',
            valueType: 'formList',
            columns,
            span: 3,
            render: (text, row, _, action) => {
              return (
                <AutoTables
                  columns={columnsc?.map((it) => ({
                    ...it,
                    hideInSearch: true,
                  }))}
                  dataSource={drawer?.item?.maintainTaskItemList}
                />
              );
            },
          },
        ];
      } else if (drawer?.type == 'wcby') {
        const columnsc = [
          {
            title: '保养项目',
            dataIndex: 'maintainItemName',
            key: 'maintainItemName',
            hideInForm: true,
          },
          {
            title: '部位',
            dataIndex: 'maintainPosition',
            key: 'maintainPosition',
            hideInForm: true,
          },
          {
            title: '保养方法',
            dataIndex: 'maintainMethod',
            key: 'maintainMethod',
            hideInForm: true,
          },
          {
            title: '结果',
            dataIndex: 'result',
            key: 'result',
            hideInForm: true,
            render: (text, row, _, action) => {
              return row.judgeType == 1 ? (
                <Radio.Group
                  onChange={(e) => {
                    row.result = e.target.value;
                  }}
                >
                  <Radio value={1}>正常</Radio>
                  <Radio value={2}>异常</Radio>
                </Radio.Group>
              ) : (
                <InputNumber
                  onChange={(value) => {
                    row.result = value;
                  }}
                />
              );
            },
          },
          {
            title: '下限值',
            dataIndex: 'lowerLimit',
            key: 'lowerLimit',
            hideInForm: true,
          },
          {
            title: '上限值',
            dataIndex: 'upperLimit',
            key: 'upperLimit',
            hideInForm: true,
          },
          {
            title: '备注',
            dataIndex: 'remark',
            key: 'remark',
            hideInForm: true,
            render: (text, row, _, action) => {
              return (
                <Input
                  onChange={(e) => {
                    row.remark = e.target.value;
                  }}
                />
              );
            },
          },
        ];
        return [
          {
            title: '工单信息',
            valueType: 'split',
          },
          {
            title: '保养单号',
            dataIndex: 'taskNo',
            key: 'taskNo',
          },
          {
            title: '创建时间',
            dataIndex: 'createTime',
            key: 'createTime',
          },
          {
            title: '工单状态',
            dataIndex: 'taskStatusName',
            key: 'taskStatusName',
          },
          {
            title: '保养计划单号',
            dataIndex: 'maintainNo',
            key: 'maintainNo',
          },
          {
            title: '设备编号',
            dataIndex: 'equipmentNo',
            key: 'equipmentNo',
          },
          {
            title: '设备名称',
            dataIndex: 'equipmentName',
            key: 'equipmentName',
          },
          {
            title: '设备型号',
            dataIndex: 'equipmentModelName',
            key: 'equipmentModelName',
          },
          {
            title: '保养类型',
            dataIndex: 'maintainTypeName',
            key: 'maintainTypeName',
          },
          {
            title: '保养频次',
            dataIndex: 'maintainFrequencyName',
            key: 'maintainFrequency',
          },
          {
            title: '保养截止日期',
            dataIndex: 'planMaintainDate',
            key: 'planMaintainDate',
          },
          {
            title: '接单时间',
            dataIndex: 'maintainStartTime',
            key: 'maintainStartTime',
            span: 2,
          },
          {
            title: '保养项目',
            valueType: 'split',
          },
          {
            dataIndex: 'itemList',
            key: 'itemList',
            valueType: 'formList',
            columns,
            span: 3,
            render: (text, row, _, action) => {
              return (
                <AutoTables
                  columns={columnsc?.map((it) => ({
                    ...it,
                    hideInSearch: true,
                  }))}
                  dataSource={drawer?.item?.maintainTaskItemList}
                />
              );
            },
          },
        ];
      }
    }
  }, [drawer?.item, activeTabKey]);

  const order = (text, row, _, action) => {
    return (
      <PremButton
        pop={{
          title: '是否接单?',
          okText: '确认',
          cancelText: '取消',
          onConfirm: async () => {
            await runAsync({
              url: '/maintain/umMaintainTask/orderReceiving',
              params: { id: row?.id },
            });
          },
        }}
        btn={{
          size: 'small',
        }}
      >
        接单
      </PremButton>
    );
  };
  const close = (text, row, _, action) => {
    return (
      <PremButton
        pop={{
          title: '是否关单?',
          okText: '确认',
          cancelText: '取消',
          onConfirm: async () => {
            await runAsync({
              url: '/maintain/umMaintainTask/customsDeclaration',
              params: { id: row?.id },
            });
          },
        }}
        btn={{
          size: 'small',
          type: 'danger',
        }}
      >
        关单
      </PremButton>
    );
  };
  const finish = (text, row, _, action) => {
    return (
      <PremButton
        btn={{
          size: 'small',
          onClick: () => {
            doFetch({
              url: '/maintain/umMaintainTask/queryById',
              params: { id: row.id },
            }).then((res) => {
              if (res.code == '0000') {
                setdrawer((s) => ({
                  ...s,
                  open: true,
                  item: {
                    ...res.data?.data,
                    id: row.id,
                  },
                  title: '完成保养',
                  val: 'detailaddon',
                  type: 'wcby',
                }));
              }
            });
          },
        }}
      >
        完成保养
      </PremButton>
    );
  };

  const remove = (text, row, _, action) => {
    return (
      <PremButton
        pop={{
          title: '是否删除?',
          okText: '确认',
          cancelText: '取消',
          onConfirm: () => {
            run({ url: pathconfig?.delete || '/delete', params: { id: row?.id } });
          },
        }}
        btn={{
          size: 'small',
          type: 'danger',
        }}
      >
        删除
      </PremButton>
    );
  };

  const verify = (text, row, _, action) => {
    return (
      <PremButton
        btn={{
          size: 'small',
          onClick: () => {
            doFetch({
              url: '/maintain/umMaintainTask/queryById',
              params: { id: row.id },
            }).then((res) => {
              if (res.code == '0000') {
                setdrawer((s) => ({
                  ...s,
                  open: true,
                  item: {
                    ...res.data?.data,
                    id: row.id,
                  },
                  title: '验证',
                  val: 'detailaddon',
                  type: 'yz',
                }));
              }
            });
          },
        }}
      >
        验证
      </PremButton>
    );
  };

  const columns = useMemo(() => {
    let defcolumn = getcolumns(setdrawer).filter((it) => it.key == activeTabKey)[0]?.columns;
    let defpath = getcolumns(setdrawer).filter((it) => it.key == activeTabKey)[0]?.pathconfig ?? {};
    return activeTabKey == 1
      ? defcolumn.concat({
          title: '操作',
          valueType: 'option',
          width: 150,
          render: (text, row, _, action) => [
            order(text, row, _, action),
            close(text, row, _, action),
            finish(text, row, _, action),
            verify(text, row, _, action),
          ],
        })
      : defcolumn;
  }, [activeTabKey]);

  const pathconfig = useMemo(() => {
    let defpath = getcolumns(setdrawer).filter((it) => it.key == activeTabKey)[0]?.pathconfig ?? {};
    return defpath;
  }, [activeTabKey]);

  const seletType = {
    wcby: (
      <div>
        <PremButton
          btn={{
            type: 'primary',
            loading,
            onClick: () => {
              let maintainTaskItemList = drawer?.item?.item?.map((it) => {
                if (it.judgeType == 1) {
                  return {
                    id: it.id,
                    remark: it.remark,
                    judgeResultType: it.result,
                  };
                } else {
                  return {
                    id: it.id,
                    remark: it.remark,
                    qualitativeJudgeResult: it.result,
                  };
                }
              });
              run({
                url: '/maintain/umMaintainTask/finishMaintainTask',
                params: { id: drawer?.item?.id, maintainTaskItemList },
              });
            },
          }}
        >
          提交
        </PremButton>
      </div>
    ),
    yz: (
      <div>
        <Divider />
        <InitForm
          fields={[
            {
              title: '验证结果',
              valueType: 'select',
              dataIndex: 'approveStatus',
              key: 'approveStatus',
              formItemProps: {
                rules: [
                  {
                    required: true,
                    message: '此项为必填项',
                  },
                ],
              },
              options: [
                {
                  value: '1',
                  label: '通过',
                },
                {
                  value: '2',
                  label: '不通过',
                },
              ],
            },
            {
              title: '备注',
              valueType: 'textarea',
              dataIndex: 'approveContent',
              key: 'approveContent',
            },
          ]}
          onFinish={(vals) => {
            run({
              url: '/maintain/umMaintainTask/verification',
              params: { ...vals, id: drawer?.item?.id },
            });
          }}
        />
      </div>
    ),
  };

  const DetailLine = () => {
    return (
      <>
        <DetailNode
          path="/maintain/umMaintainTaskOperation/queryDetailList"
          params={
            activeTabKey == 3
              ? { maintainTaskId: drawer?.item?.equipmentMaintainTaskId }
              : { maintainTaskId: drawer?.item?.id }
          }
          titleColumns={
            activeTabKey == 3
              ? [
                  {
                    title: '保养单号',
                    dataIndex: 'taskNo',
                    key: 'taskNo',
                  },
                  {
                    title: '创建时间',
                    dataIndex: 'createTime',
                    key: 'createTime',
                  },
                  {
                    title: '工单状态',
                    dataIndex: 'taskStatusName',
                    key: 'taskStatusName',
                  },
                  {
                    title: '保养计划单号',
                    dataIndex: 'maintainNo',
                    key: 'maintainNo',
                  },
                  {
                    title: '关单时间',
                    dataIndex: 'customsTime',
                    key: 'customsTime',
                  },
                ]
              : [
                  {
                    title: '保养单号',
                    dataIndex: 'taskNo',
                    key: 'taskNo',
                  },
                  {
                    title: '创建时间',
                    dataIndex: 'createTime',
                    key: 'createTime',
                  },
                  {
                    title: '工单状态',
                    dataIndex: 'taskStatusName',
                    key: 'taskStatusName',
                  },
                  {
                    title: '保养计划单号',
                    dataIndex: 'maintainNo',
                    key: 'maintainNo',
                  },
                ]
          }
          detailKey="maintainTaskItemList"
          columns={getDetailColumns}
        />
      </>
    );
  };

  return (
    <div style={{ position: 'relative' }}>
      <AutoTable
        pagetitle={<h3 className="page-title">保养工单</h3>}
        columns={columns}
        path={pathconfig?.list || '/ngic-auth/sysUser/query/page'}
        actionRef={actionRef}
        pageextra={pathconfig?.enableadd ? 'add' : null}
        resizeable={false}
        addconfig={{
          // access: 'sysDepartment_save',
          btn: {
            disabled: false,
            onClick: () => {
              setdrawer((s) => ({
                ...s,
                open: true,
                item: null,
                title: '新增',
                val: 'add',
              }));
            },
          },
        }}
        tabList={getcolumns()}
        activeTabKey={activeTabKey}
        onTabChange={(key) => {
          setactiveTabKey(key);
        }}
      />

      <DrawerPro
        fields={detailaddonColumns}
        detailpath={pathconfig?.detail || null}
        detailData={drawer?.item}
        defaultFormValue={drawer?.item}
        params={{ id: drawer?.item?.id }}
        formRef={formRef}
        placement="right"
        onClose={() => {
          setdrawer((s) => ({
            ...s,
            open: false,
          }));
        }}
        {...drawer}
        onFinish={(vals) => {
          if (drawer?.val == 'add') {
            run({ url: pathconfig?.add || '/add', params: { ...vals } });
          } else if (drawer?.val == 'edit') {
            run({ url: pathconfig?.edit || '/edit', params: { ...vals, id: drawer?.item?.id } });
          }
        }}
      >
        {drawer?.val == 'only' ? (
          <DetailLine />
        ) : drawer?.val == 'detailaddon' ? (
          seletType[drawer?.type]
        ) : null}
      </DrawerPro>
    </div>
  );
}

export default WorkOrder;
